@@ -184,7 +184,7 @@ impl HyperHandler {
             }
             if res.body.is_none() && has_error {
                 for catcher in catchers.iter().chain(DEFAULT_CATCHERS.iter()) {
-                    if catcher.catch(&req, &mut res) {
+                    if catcher.catch(&req, &depot, &mut res) {
                         break;
                     }
                 }
@@ -224,7 +224,6 @@ impl hyper::service::Service<hyper::Request<hyper::body::Body>> for HyperHandler
 #[cfg(test)]
 mod tests {
     use crate::prelude::*;
-    use crate::routing::FlowCtrl;
 
     #[tokio::test]
     async fn test_service() {
